# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields


class res_company(osv.Model):

    """Extend the res.company model to meet AXIMA requirements"""
    _inherit = 'res.company'

    def _get_city2(self, cr, uid, ids, fieldname, args, context=None):
        """
        city: extract of that is before 'CEDEX'
        """
        def remove_cedex(city):
            tokens = (' CEDEX', ' Cedex', ' cedex', )
            cedex_index = -1
            for t in tokens:
                cedex_index = city.find(t)
                if cedex_index >= 0:
                    break
            if cedex_index >= 0:
                if cedex_index == 0:
                    city = ''
                else:
                    city = city[0:cedex_index]
            return city.rstrip(' ')
        
        if isinstance(ids, (int, long)):
            ids = [ids]
        res = {}
        for r in self.read(cr, uid, ids, ['city'], context=context):
            res[r['id']] = remove_cedex(r['city'])
        return res

    def _get_region_selection(self, cursor, user_id, context=None):
        """Return the tuple containing valid values for the region field"""
        regions = (u"CENTRE", u"Dpt Industries", u"EST",  u"NORD", u"OUEST",
                   u"PARIS – EST",  u"SUD EST", u"SUD OUEST")
        selection = []
        for region in regions:
            selection.append((region, region))
        # (('key_or_value', 'string_to_display'), ... )
        return tuple(selection)

    _post_sales_person_label = u"Responsable du service après-vente"
    _post_sales_as_label = u"Assistants du service après-vente"

    _columns = {
                'sap_company_code': fields.char(u"Référence SAP",
                                                size=10,
                                                required=True,
                                                select=1),
                'office_code': fields.char(u"Code agence",
                                                  size=10,
                                                  required=True,
                                                  select=1),
                'region': fields.selection(_get_region_selection,
                                           u"Région",
                                           required=False
                                           # required=True,
                                           ),
                'person_in_charge': fields.many2one('res.partner',
                                                    u"Responsable de l’agence",
                                                    required=True),
                'post_sales_person': fields.many2one('res.partner',
                                                     _post_sales_person_label,
                                                     required=False),
                'post_sales_assistants': fields.many2many('res.partner',
                                                          'company_psa_rel',
                                                          'company_id',
                                                          'partner_id',
                                                          _post_sales_as_label,
                                                          required=False
                                                          ),
                'company_assitants': fields.many2many('res.partner',
                                                      'company_ca_rel',
                                                      'company_id',
                                                      'partner_id',
                                                      u"Assistants d'agence",
                                                      required=False,
                                                      # required=True, # les
                                                      # données sont incomplète
                                                      ),
                'region_director': fields.many2one('res.partner',
                                                   u"Directeur de région",
                                                   # required=True # peut-être,
                                                   # mais pas avec l’état
                                                   # actuel de la base
                                                   required=False,
                                                   ),
                'city2': fields.function(_get_city2, type='char', string="Ville", method=True),
    }
 
    _defaults = {
        'sap_company_code': False,
        'office_code': False,
        'region': False,
        'person_in_charge': False,
        'post_sales_person': False,
        'post_sales_assistants': False,
        'company_assitants': False,
        'region_director': False,
    }
