# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields
from openerp import tools
from openerp.tools import image_resize_image
import os


class res_company(osv.osv):

    '''Extend the res_company model to provide default report hearder and
       footer values matching AXIMA requirement.'''
    _inherit = 'res.company'

    def _get_footer(self, cr, uid, ids):
        '''Renvoie le texte de pied de page définie dans le fichier
        correspondant ou rien s'il n'existe pas.
        '''
        try:
            footer_file = tools.file_open(
                os.path.join('axima', 'report',
                             'corporate_rml_footer.txt'))
            try:
                return footer_file.read()
            finally:
                footer_file.close()
        except:
            return ""

    def _get_header(self, cr, uid, ids):
        '''envoie le texte d'entête définie dans le fichier correspondant ou
        rien s'il n'existe pas.
        '''
        try:
            header_file = tools.file_open(
                os.path.join('axima', 'report',
                             'corporate_rml_header.rml'))
            try:
                return header_file.read()
            finally:
                header_file.close()
        except:
            # return self._header_a4
            return super(res_company, self)._header_a4

    def _get_logo(self, cr, uid, ids):
        return open(os.path.join(tools.config['root_path'],
                                 os.pardir, os.pardir, 'addons',
                                 'axima', 'images', 'AXIMA.jpg'),
                    'rb').read().encode('base64')

    def _get_logo_web(self, cr, uid, ids, _field_name, _args, context=None):
        '''On surcharge pour modifier la taille, ce afin d'éviter d'obtenir
        un énorme logo pixilisé
        '''
        result = dict.fromkeys(ids, False)
        for record in self.browse(cr, uid, ids, context=context):
            size = (180, None)
            size = (90, None)
            result[
                record.id] = image_resize_image(
                record.partner_id.image,
                size)
        return result

    def onchange_paper_format(self, cr, uid, ids, paper_format, context=None):
        '''Est appelé par le formulaire de création d'entreprise, cette
        surcharge évite que la valeur par défaut de rml_header définie dans le
        présent module ne soit écrasée par celle de _header_a4 de la classe
        parente.
        '''
        if paper_format == 'us_letter':
            return {'value': {'rml_header':
                              'Error: us letter format not supported'}}
        return {'value': {'rml_header': self._get_header(cr, uid, ids)}}

    _defaults = {
        'rml_footer_readonly': True,
        'rml_footer': _get_footer,
        'rml_header': _get_header,
        'logo': _get_logo,
        # 'rml_header1': "Axima réfrigération : le référent du froid",
        # 'rml_header2': "exemple",
        # 'rml_header3': "exemple",
    }
 
res_company()
