"""
DEPLOY COMPANIES CASCADE
"""
import xmlrpclib
import csv

TEST_MODE = False
FILE_NAME = '../data/T_Agences.csv'

username = 'admin'  # the user
pwd = 'A$mixa!'  # the password of the user
dbname = 'AXIMA'  # the database

# Get the uid
sock_common = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/common')
uid = sock_common.login(dbname, username, pwd)

# replace localhost with the address of the server
sock = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/object')

errors = []
ids_map = {}  # { 'model1': { 'name1': id, ... }, ... }
xmlid_ids_map = {}


"""
HELPERS
"""
class DeployCompanyException(Exception):
    pass

def get_id_from_name(model, name):
    if not model in ids_map:
        ids_map[model] = {}
    id = ids_map[model].get(name, False)
    if id:
        return id
    domain = [('name', '=', name)]
    id = sock.execute(dbname, uid, pwd, model, 'search', domain)
    if id:
        id = id[0]
    ids_map[model][name] = id
    return id
    
def get_id_from_xmlid(xmlid, module_dir='axima'):
    id = xmlid_ids_map.get(xmlid, False)
    if id:
        return id
    obj_ref = sock.execute(dbname, uid, pwd, 'ir.model.data',
        'get_object_reference', module_dir, xmlid)
    if obj_ref:
        xmlid_ids_map[xmlid] = obj_ref[1]
        return obj_ref[1]
    xmlid_ids_map[xmlid] = False
    return False
    
def add_err(index, description):
    msg = "! ERROR / line %d / %s" % (index, description, )
    errors.append(msg)
    
"""
SCRIPT METHOD
"""
def do_import(data):
    if TEST_MODE:
        print '** TEST MODE **'
    file = open(FILE_NAME, "rb")
    try:
        reader = csv.DictReader(file, delimiter=';', quoting=csv.QUOTE_NONE)
        index = 1
        for row in reader:
            import_company(row, index, data)
            if TEST_MODE:
                break  # only first line processed for test mode
            index += 1
    finally:
        file.close()
        
def import_company(row, index, data):  
    company_fields = [
        'sap_company_code',
        'office_code',
        'name',
        'street',
        'street2',
        'zip',
        'city',
        'phone',
        'fax',
        #'region',
    ]
    key = 'office_code'
    name_field = 'name'
    parent_id = 1
    
    partned_fields = [
        'sap_company_code',
        'office_code',
        'name',
        'street',
        'street2',
        'zip',
        'city',
        'phone',
        'fax',
    ]
    
    header_content = data.get('header_content', False)
    logo = data.get('logo', False)
     
    # checks
    key_val = row.get(key, False)
    if not key_val:
        add_err(index, "'%s' key field value is not defined" % (key, ))
        
    # set vals
    company_vals = {}
    partner_vals = {}
    
    name = row.get(name_field, '')
    
    # company vals
    for f in company_fields:
        company_vals[f] = row[f]
    company_vals['parent_id'] = parent_id
    display_vals = company_vals.copy()
    for d in data:
        company_vals[d] = data[d]
    
    # partner vals
    for f in partned_fields:
        partner_vals[f] = row[f]
    partner_vals['logo'] = data.get('logo', False)
    print "------------------------------"
    print "COMPANY"
    print "%s - %s" % (company_vals[key], name, )
    print display_vals  # displayed without huge data fields
    
    # object creation/update cascade
    domain = [('parent_id', '!=', False), (key, '=', key_val)]
    ids = sock.execute(dbname, uid, pwd, 'res.company', 'search', domain)
    if ids:
        # update company
        # !!! TODO: for now only creation of new companies
        print 'ALREADY CREATED'
        pass
    else:
        # create company and related objects cascade
        print 'CREATE'
        
        # COMPANY
        company_id = sock.execute(dbname, uid, pwd, 'res.company', 'create',
            company_vals)
            
        # SALE TEAM
        vals = {
            'code': key_val,
            'name': name,
        }
        print "SALE TEAM"
        print vals
        sale_team_id = sock.execute(dbname, uid, pwd, 'crm.case.section',
            'create', vals)
        
        # PARTNER
        # get partner created by company creation
        r_comp = sock.execute(dbname, uid, pwd, 'res.company', 'read',
            [company_id], ['partner_id'])
        if not r_comp:
            add_err(index, "company's partner not found: %s %s" % (key_val,
                name))
            return
        partner_id = r_comp[0]['partner_id'][0]
        partner_vals.update({
            'section_id': sale_team_id,
            'supplier': False,
            'customer': False,
            'is_company': True,
            'sap_client_code': '',
        })
        print "PARTNER"
        display_vals = partner_vals.copy()
        if 'logo' in display_vals:
            del display_vals['logo']
        print display_vals
        sock.execute(dbname, uid, pwd, 'res.partner', 'write', partner_id,
            partner_vals)
            
        # WAREHOUSE
        vals = {
            'company_id': company_id,
            'partner_id': partner_id,
            'lot_input_id': 12,  # stock
            'lot_output_id': 11,
            'lot_stock_id': 12,
            'name': name,
        }
        print "WAREHOUSE"
        print vals
        warehouse_id = sock.execute(dbname, uid, pwd, 'stock.warehouse',
            'create', vals)
        
        # SHOP
        vals = {
            'company_id': company_id,
            'warehouse_id': warehouse_id,
            'payment_default_id': 3,  # '30 jours net'
            'pricelist_id': 1,  # default pricelist
            'name': name,
        }
        print "SHOP"
        print vals
        sock.execute(dbname, uid, pwd, 'sale.shop', 'create', vals)
            
        # PRODUCT CATEGORY
        parent_categ_id = get_id_from_xmlid('axima_product_category_agence')
        vals = {
            'parent_id': parent_categ_id,  # All articles
            'type': 'normal',
            'name': name,
            'sap_company_code': company_vals['sap_company_code'],
            'office_code': key_val,
        }
        print "PRODUCT CATEGORY"
        print vals
        sock.execute(dbname, uid, pwd, 'product.category', 'create',
            vals)
            

"""
SCRIPT
"""
data = {} 

# some HQ template datas (logo, rml header/footer)
fields = ['logo', 'rml_header', 'rml_footer']
logo = False
r = sock.execute(dbname, uid, pwd, 'res.company', 'read', 1, fields)
if r:
    for f in fields:
        data[f] = r[f]
    if r['logo']:
        print 'LOGO is found'
    
do_import(data)

if errors:
    for err in errors:
        print err
    print "%d errors" % (len(errors), )
# EOF
