# -*- coding:utf-8 -*-
"""
IMPORT CUSTOMERS
"""
import xmlrpclib
import csv

import unicode_tools

TEST_MODE = False
FILE_NAME = '../data/tbl_Clients_agence.csv'

username = 'admin'  # the user
pwd = 'A$mixa!'  # the password of the user
dbname = 'AXIMA'  # the database

# Get the uid
sock_common = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/common')
uid = sock_common.login(dbname, username, pwd)

# replace localhost with the address of the server
sock = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/object')

errors = []
ids_map = {}  # { 'model1': { 'name1': id, ... }, ... }


"""
HELPERS
"""
class ImportCustomerException(Exception):
    pass

def get_id_from_name(model, name):
    if not model in ids_map:
        ids_map[model] = {}
    id = ids_map[model].get(name, False)
    if id:
        return id
    domain = [('name', '=', name)]
    id = sock.execute(dbname, uid, pwd, model, 'search', domain)
    if id:
        id = id[0]
    ids_map[model][name] = id
    return id
    
def add_err(index, description):
    msg = "! ERROR / line %d / %s" % (index, description, )
    errors.append(msg)
    
"""
SCRIPT METHOD
"""
def do_import(res):
    if TEST_MODE:
        print '** TEST MODE **'
    file = open(FILE_NAME, "rb")
    try:
        reader = csv.DictReader(file, delimiter=';', quoting=csv.QUOTE_NONE)
        index = 1
        for row in reader:
            import_customer(row, index, res)
            if TEST_MODE:
                break  # only first line processed for test mode
            index += 1
    finally:
        file.close()
        
def import_customer(row, index, res):  
    fields = [
        'sap_client_code',
        'siret',
        'sap_company_code',
        'search_criterion',
        'name',
        'name2',
        'street',
        'street2',
        'zip',
        'city',
        'country_code',
        'purchaser',
        'otp',
        'tech_post',
        #'method_of_payment,
        'phone',
        'fax',
        'email',
    ]
    
    defaults = {
        'customer': True,
        'supplier': False,
        'is_company': False,
    }
    
    key = 'sap_client_code'
    comp_key = 'sap_company_code'
    name_field = 'name'
     
    # checks
    key_val = row.get(key, False)
    if not key_val:
        add_err(index, "'%s' '%s' key field value is not defined" % (key, key_val))
    comp_key_val = row.get(comp_key, False)
    if not key_val:
        add_err(index, "'%s' '%s' company key field value is not defined" % (comp_key, comp_key_val, ))
        
    # set vals
    vals = {}  
    name = row.get(name_field, '')
    for f in fields:
        vals[f] = row[f]
    for d in defaults:
        vals[d] = defaults[d]
    vals_clean(vals)
    print "%d: %s" % (index, vals, )
    
    # company link (customer of company)
    domain = [(comp_key, '=', comp_key_val)]
    comp_ids = sock.execute(dbname, uid, pwd, 'res.company', 'search',
        domain)
    if comp_ids:
        r_comp = sock.execute(dbname, uid, pwd, 'res.company', 'read',
            comp_ids[0], ['office_code', 'name'])
        if r_comp:
            vals['office_code'] = r_comp['office_code']
            vals['company_id'] = r_comp['id']
        else:
            add_err(index, "'%s' (%d) error read company res_partner entry" % (comp_key_val, company_partner_ids[0], ))
    else:
        add_err(index, "'%s' error searching company res_partner entry" % (comp_key_val, ))
        
    domain = [(key, '=', key_val), ('customer', '=', True)]
    ids = sock.execute(dbname, uid, pwd, 'res.partner', 'search', domain)
    if ids:
        print 'update'
        del vals[key]
        sock.execute(dbname, uid, pwd, 'res.partner', 'write', ids[0], vals)
        res['updated'] += 1
    else:
        print 'create'
        sock.execute(dbname, uid, pwd, 'res.partner', 'create', vals)
        res['created'] += 1

def vals_clean(vals):
    if '' in vals:
        del vals['']
    # cleaning name with severals weird tokens
    encode_fields = ['name', 'name2', 'search_criterion', 'street', 'street2']
    trim_strs = [ '"' ]
    trim_strs = [ '"', '\xc2', '\xa0', '\x1a', '\xb0', '0xc3']
    for f in encode_fields:
        for ts in trim_strs:
            vals[f] = vals[f].replace(ts, '')
        vals[f] = unicode_tools.ustr(vals[f])

"""
SCRIPT
"""
res = {'created': 0, 'updated': 0}
do_import(res)

print "\ncreated: %d\nupdated: %d\n" % (res['created'], res['updated'], )

if errors:
    for err in errors:
        print err
    print "%d errors" % (len(errors), )
# EOF
