# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 2014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""
PRODUCT IMPORT
"""
import xmlrpclib
import csv

INITIAL_IMPORT = False  # only record creations
TEST_MODE = False
FILES_SUFIX = [ 'Fritec', 'Le_Froid', 'Mercuriale', ]
DEFAULT_PRODUCT_STATE = 'sellable'

username = 'admin'  # the user
pwd = 'A$mixa!'  # the password of the user
dbname = 'AXIMA'  # the database

# Get the uid
sock_common = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/common')
uid = sock_common.login(dbname, username, pwd)

# replace localhost with the address of the server
sock = xmlrpclib.ServerProxy('http://localhost:8069/xmlrpc/object')

errors = []
ids_map = {}  # { 'model1': { 'name1': id, ... }, ... }
xmlid_ids_map = {}  # { 'xmlid1': id, ... }

"""
PRODUCT UOM MAPPING
key: import file value, value: OpenERP English name
"""
product_uom_map = {
    '/PC': 'Unit(s)',  # note: 'PCE' in FR
    'HRE': 'Hour(s)',  # note: 'HRE' in FR
    'KG': 'kg',
    'KM': 'km',
    'M': 'm',
    'M3': 'm3',
}
product_uom_default = product_uom_map['/PC']

res = {}

"""
HELPERS
"""
class ImportProductException(Exception):
    pass

def axima_product_import_rule_set_active(active):
    xmlid = 'axima_record_rule_product_by_import'
    rule_id = get_id_from_xmlid(xmlid)
    if not rule_id:
        raise ImportProductException("can not found ir.rule '%s'" % (xmlid, ))
    vals = {'active': active}
    sock.execute(dbname, uid, pwd, 'ir.rule', 'write', rule_id, vals)
    
def import_file(sufix, res):
    file_name = '../data/Articles_%s.csv' % (sufix, )
    print "\n===========\nIMPORT FILE %s \n===========\n" % (file_name, )
    if TEST_MODE:
        print '** TEST MODE **'
    file = open(file_name, "rb")
    try:
        reader = csv.DictReader(file, delimiter=';', quoting=csv.QUOTE_NONE)
        index = 1
        res[sufix] = {'created': 0, 'updated': 0}
        for row in reader:
            import_row(row, index, sufix, res)
            index += 1
            if TEST_MODE:
                break  # only first line processed for test mode
    finally:
        file.close()
        
def import_row(row, index, sufix, res):
    # checks
    name = row.get('name', False)
    if not name:
        add_err(index, suffix, 'no name')
    code = row.get('default_code', False)
    if not code:
        add_err(index, suffix, 'no default_code')

    # get product category id ('categ_id')
    categ_id = get_category_id(sufix)
    if not categ_id:
        add_err(index, sufix,
            "categ_id not found for category '%s'" % category)
            
    # product exists already ?
    if INITIAL_IMPORT:
        product_id = False
    else:
        domain = [('default_code', '=', code)]
        product_id = sock.execute(dbname, uid, pwd, 'product.product', 'search',
        domain)
        
    # set vals
    vals = row.copy()
    # product flagged as imported...
    # ...for axima_record_rule_product_by_import record rule
    vals_clean(sufix, vals)
    vals['state'] = DEFAULT_PRODUCT_STATE
    vals['import_flag'] = 'imported'
    vals['sap_product_code'] = code  # = default_code
    vals['categ_id'] = categ_id
    vals_encode_floats(vals, ['list_price', 'standard_price'])
    """
    if not 'standard_price' in vals and 'list_price' in vals:
        # default standard_price (pur price) of list_price (sale price)
        vals['standard_price'] = vals['list_price']
    """
    vals_set_uom(vals)
    print vals
    
    if product_id:
        sock.execute(dbname, uid, pwd, 'product.product', 'write', product_id,
            vals)
        print 'updated'
        res[sufix]['updated'] += 1
    else:
        sock.execute(dbname, uid, pwd, 'product.product', 'create', vals)
        print 'created'
        res[sufix]['created'] += 1

def vals_clean(sufix, vals):
    if '' in vals:
        del vals['']
    if 'PASS' in vals:
        del vals['PASS']
    if sufix == 'Mercuriale':
        # cleaning names with severals weird " (from original 'Mercuriale' xls)
        vals['name'] = vals['name'].replace('"', '')
    
def vals_encode_floats(vals, keys):
    for k in keys:
        if k in vals:
            if vals[k]:
                value = vals[k]
                if value == 'NC':
                    del vals[k]
                else:
                    trim_strs = [ ' ', '\xc2', '\xa0', ]
                    for ts in trim_strs:
                        value = value.replace(ts, '')
                    if ',' in value:
                        value = float(value.replace(',', '.'))
                    elif not '.' in vals[k]:
                        value = float(value + '.0')
                    vals[k] = value
            else:
                del vals[k]
            
def vals_set_uom(vals):
    uom_name = False
    if 'UOM' in vals:
        uom_name = product_uom_map.get(vals['UOM'], False)
        del vals['UOM']
    if not uom_name:
        uom_name = product_uom_default
    uom_id = get_id_from_name('product.uom', uom_name)
    if uom_id:
        vals['uom_id'] = uom_id
        vals['uom_po_id'] = uom_id
        
def get_id_from_xmlid(xmlid, module_dir='axima'):
    id = xmlid_ids_map.get(xmlid, False)
    if id:
        return id
    obj_ref = sock.execute(dbname, uid, pwd, 'ir.model.data',
        'get_object_reference', module_dir, xmlid)
    if obj_ref:
        xmlid_ids_map[xmlid] = obj_ref[1]
        return obj_ref[1]
    xmlid_ids_map[xmlid] = False
    return False
            
def get_category_id(sufix):
    category_name = sufix.replace('_', ' ')
    return get_id_from_name('product.category', category_name)
       
def get_id_from_name(model, name):
    if not model in ids_map:
        ids_map[model] = {}
    id = ids_map[model].get(name, False)
    if id:
        return id
    domain = [('name', '=', name)]
    id = sock.execute(dbname, uid, pwd, model, 'search', domain)
    if id:
        id = id[0]
    ids_map[model][name] = id
    return id
    
def add_err(index, suffix, description):
    msg = "! ERROR / file %s / line %d / %s" % (suffix, index, description, )
    errors.append(msg)
    
"""
SCRIPT
"""
# rule not used now, but leave here for memo
#axima_product_import_rule_set_active(False)  # rule off in order to import!
 
for sufix in FILES_SUFIX:
    import_file(sufix, res)
    
#axima_product_import_rule_set_active(True)  # rule on again!

for s in FILES_SUFIX:
    r = res.get(s, {})
    print "%s" % (s, )
    print "%d lines created" % (r.get('created', 0), )
    print "%d lines updated" % (r.get('updated', 0), )

if errors:
    for err in errors:
        print err
    print "%d errors" % (len(errors), )
# EOF
